
// ==================================================================================
//  This program reads a BNO055 sensor and sends AccX, AccY, AccZ to IotHAL
//  Values are sent to IotHAL as 16 bits from 0 to 65535
// ----------------------------------------------------------------------------------
//  SIMPLE VERSION - Without libraries
// ==================================================================================

// ==================================================================================
//   ESP32 to BNO055
// ==================================================================================
//   ESP32 Pin      BNO055 
//  ---------------------------------------------------------------------------------
//     GND          GND 
//     +5V          VIN
//     27           SDA        
//     14           SCL            
// ----------------------------------------------------------------------------------   

// ==================================================================================
//   ESP32 TO IOT-HAL
// ==================================================================================
//   ESP32 Pin     Signal type                        IotHal Pin configuration 
//  ---------------------------------------------------------------------------------
//      36          X (acceleration right-left)         GenericIn16
//      39          Y (acceleration front-rear)         GenericIn16
//      25 (or 34)  Z (acceleration up-down)            GenericIn16
// ----------------------------------------------------------------------------------     

#include "IotModule\IotModule.h"
#include "Utility/ThereminoFilters.h"
#include <Wire.h>

// ===================================
//  IotModule object, do not remove
// ===================================
cIotModule IotModule;
// ===================================

// -------------------------------------------------------- eventually modify pins
byte pinX = 36;
byte pinY = 39;
byte pinZ = 25;    // 25 on Pico - 34 on Wroom
byte pinSDA = 27;
byte pinSCL = 14;
byte pinTest = 21;

// ------------------------------------------------------------------------------
//  Filters to transform accelerometers to velocimeters (see documentation)
// ------------------------------------------------------------------------------
//  With 4.5 Hz the resulting signal is similar to a 4.5 Hz Geophone 
//  You could change HP and LP freq. to get different behaviours
//  Or you could cmment the following USE_FILTERS to get unfiltered data
// ------------------------------------------------------------------------------
#define USE_FILTERS
const float HiPassFreq  = 4.5;
const float LowPassFreq = 4.5;
// -------------------------------------------------------- filters declarations 
Filter hipass1(HiPassFreq, true);
Filter hipass2(HiPassFreq, true);
Filter hipass3(HiPassFreq, true);
Filter lopass1(LowPassFreq, false);
Filter lopass2(LowPassFreq, false);
Filter lopass3(LowPassFreq, false);

// -------------------------------------------------------- do not modify following lines
float x, y, z;

// ------------------------------------------------------------------------------
//   BNO055 functions
// ------------------------------------------------------------------------------
#define BNO055_ID 0xA0
#define BNO055_ADDRESS 0x29
#define BNO055_OPR_MODE_ADDR 0x3D
#define OPERATION_MODE_CONFIG 0x00
#define OPERATION_MODE_ACCONLY 0x01
#define BNO055_SYS_TRIGGER_ADDR 0x3F
#define BNO055_CHIP_ID_ADDR 0x00
#define BNO055_PWR_MODE_ADDR 0x3E
#define POWER_MODE_NORMAL 0x00
#define BNO055_PAGE_ID_ADDR 0x07
#define BNO055_ACCEL_DATA_X_LSB_ADDR 0x08

void write8(uint8_t reg, uint8_t value)
{
  Wire.beginTransmission(BNO055_ADDRESS);
  Wire.write(reg);
  Wire.write(value);
  Wire.endTransmission();
}

uint8_t read8(uint8_t reg)
{
  byte value = 0;
  Wire.beginTransmission(BNO055_ADDRESS);
  Wire.write((uint8_t)reg);
  Wire.endTransmission();
  Wire.requestFrom((uint16_t) BNO055_ADDRESS, (uint8_t) 1);
  value = Wire.read();
  return value;
}

void setOpMode(uint8_t opmode)
{
  write8(BNO055_OPR_MODE_ADDR, opmode);
  delay(30);
}


// ----------------------------------------------------------- SETUP
void setup(void)
{
  // =========================================================
  //  INIT SERIAL
  // =========================================================
  Serial.begin(115200);

  // =========================================================
  //  IotModule object initialization, do not remove
  // =========================================================
  IotModule.initialize();

  // =========================================================
  //  pin mode for PIN TEST
  // =========================================================
  pinMode(pinTest, OUTPUT);

  // =========================================================
  //  Init I2C communication  (SDA / SCL / FREQUENCY)
  // =========================================================
  Wire.begin(pinSDA, pinSCL);

  // =========================================================
  //  INITIALIZE the BNO055 
  // =========================================================
  // --------------------------------------  reset and test ID
  setOpMode(OPERATION_MODE_CONFIG);
  write8(BNO055_SYS_TRIGGER_ADDR, 0x20);
  while (read8(BNO055_CHIP_ID_ADDR) != BNO055_ID)
  { 
    delay(10); 
  }
  delay(50);
  // --------------------------------------- set the normal power mode
  //setOpMode(OPERATION_MODE_CONFIG);
  write8(BNO055_PWR_MODE_ADDR, POWER_MODE_NORMAL);
  delay(10);
  write8(BNO055_PAGE_ID_ADDR, 0);
  write8(BNO055_SYS_TRIGGER_ADDR, 0);
  delay(10);
  // --------------------------------------- set chrystal (needs more than 300 mS)
  setOpMode(OPERATION_MODE_CONFIG);
  write8(BNO055_PAGE_ID_ADDR, 0);
  write8(BNO055_SYS_TRIGGER_ADDR, 0x80);
  delay(400);

  // --------------------------------------- set Bandwidth and G
  // ACC_Config reg. address = 8 / page = 1
  // ---------------------------------------
  // bits 7-6-5 = 000 = normal
  // bits 4-3-2 = 1000/500/250/125/64/32/16/8 Hz
  // bits   1-0 = 16/8/4/2 G(+/-)
  // ---------------------------------------
  // default 000 011 01 = 62.5Hz and +/-4G
  // ---------------------------------------
  setOpMode(OPERATION_MODE_CONFIG);
  write8(BNO055_PAGE_ID_ADDR, 1);
  //write8(8, 0b00011100); // 1000 Hz 2G
  write8(8, 0b00010000); // 125 Hz 2G
  //write8(8, 0b00001100); // 62.5 Hz 2G
  //write8(8, 0b00010001); // 125 Hz 4G
  //write8(8, 0b00001101); // 62.5 Hz 4G
  //write8(8, 0b00000001); //    8 Hz 4G
  //write8(8, 0b00001110); // 62.5 Hz 8G
  //write8(8, 0b00011111); // 1000 Hz 16G
  delay(10);

  // --------------------------------------- set pageID = 0 to read data 
  write8(BNO055_PAGE_ID_ADDR, 0);

  // --------------------------------------- set mode - acceleration only
  setOpMode(OPERATION_MODE_ACCONLY);
}

// ---------------------------------------------------------------------- LOOP
void loop(void)
{
  // -------------------------------------------------------- starting with register ACCEL_X_LSB
  Wire.beginTransmission(BNO055_ADDRESS);
  Wire.write(BNO055_ACCEL_DATA_X_LSB_ADDR);   
  Wire.endTransmission(false);
  // -------------------------------------------------------- request a total of 6 registers (1 to 1.5 mS)
  Wire.requestFrom((uint16_t) BNO055_ADDRESS, (uint8_t) 6, true);
  int16_t ix, iy, iz;
  ix = ((int16_t)Wire.read()) | (((int16_t)Wire.read()) << 8);
  iy = ((int16_t)Wire.read()) | (((int16_t)Wire.read()) << 8);
  iz = ((int16_t)Wire.read()) | (((int16_t)Wire.read()) << 8);

  // -------------------------------------------------------- scale the G sensitivity
  const float k = 16.0 / 32768;  // normalized from -1G to +1G
  x = k * (double)ix;
  y = k * (double)iy;
  z = k * (double)iz;
  
  // -------------------------------------------------------- HiPass and LowPass filters
  #ifdef USE_FILTERS
    x = hipass1.run(x);
    x = lopass1.run(x);
    y = hipass2.run(y);
    y = lopass2.run(y);
    z = hipass3.run(z);
    z = lopass3.run(z);
  #endif

  // -------------------------------------------------------- send X/Y/Z values to IotHAL as float numbers
  //                      normalized from 0 to 1000
  IotModule.genericWriteFloat(pinX, 500 + x * 1000);
  IotModule.genericWriteFloat(pinY, 500 + y * 1000);
  IotModule.genericWriteFloat(pinZ, 500 + z * 1000);
  
  // -------------------------------------------------------- loop time test (total = 1.5 to 2.5 mS) (248 Hz)
  //digitalWrite(pinTest, !digitalRead(pinTest));
}
